@@ -1,5 +1,5 @@
 // rak - Rakshasa's toolbox
-// Copyright (C) 2005-2006, Jari Sundell
+// Copyright (C) 2005-2007, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -42,7 +42,6 @@
 #include <cstdlib>
 #include <iterator>
 #include <locale>
-#include <stdlib.h>
 
 namespace rak {
 
@@ -192,8 +191,8 @@ OutputIterator
 copy_escape_html(InputIterator first, InputIterator last, OutputIterator dest) {
   while (first != last) {
     if (std::isalpha(*first, std::locale::classic()) ||
-	std::isdigit(*first, std::locale::classic()) ||
-	*first == '-') {
+        std::isdigit(*first, std::locale::classic()) ||
+        *first == '-') {
       *(dest++) = *first;
 
     } else {
@@ -208,6 +207,36 @@ copy_escape_html(InputIterator first, InputIterator last, OutputIterator dest) {
   return dest;
 }
 
+template <typename InputIterator, typename OutputIterator> 
+OutputIterator
+copy_escape_html(InputIterator first1, InputIterator last1, OutputIterator first2, OutputIterator last2) {
+  while (first1 != last1) {
+    if (std::isalpha(*first1, std::locale::classic()) ||
+        std::isdigit(*first1, std::locale::classic()) ||
+        *first1 == '-') {
+      if (first2 == last2) break; else *(first2++) = *first1;
+
+    } else {
+      if (first2 == last2) break; else *(first2++) = '%';
+      if (first2 == last2) break; else *(first2++) = value_to_hexchar<1>(*first1);
+      if (first2 == last2) break; else *(first2++) = value_to_hexchar<0>(*first1);
+    }
+
+    ++first1;
+  }
+
+  return first2;
+}
+
+template <typename Iterator>
+inline std::string
+copy_escape_html(Iterator first, Iterator last) {
+  std::string dest;
+  copy_escape_html(first, last, std::back_inserter(dest));
+
+  return dest;
+}
+
 template <typename Sequence>
 Sequence
 copy_escape_html(const Sequence& src) {
@@ -231,8 +260,21 @@ transform_hex(InputIterator first, InputIterator last, OutputIterator dest) {
   return dest;
 }
 
+template <typename InputIterator, typename OutputIterator> 
+OutputIterator
+transform_hex(InputIterator first1, InputIterator last1, OutputIterator first2, OutputIterator last2) {
+  while (first1 != last1) {
+    if (first2 == last2) break; else *(first2++) = value_to_hexchar<1>(*first1);
+    if (first2 == last2) break; else *(first2++) = value_to_hexchar<0>(*first1);
+
+    ++first1;
+  }
+
+  return first2;
+}
+
 template <typename Sequence>
-Sequence
+inline Sequence
 transform_hex(const Sequence& src) {
   Sequence dest;
   transform_hex(src.begin(), src.end(), std::back_inserter(dest));
@@ -240,6 +282,15 @@ transform_hex(const Sequence& src) {
   return dest;
 }
 
+template <typename Iterator>
+inline std::string
+transform_hex(Iterator first, Iterator last) {
+  std::string dest;
+  transform_hex(first, last, std::back_inserter(dest));
+
+  return dest;
+}
+
 template <typename Sequence>
 Sequence
 generate_random(size_t length) {
