@@ -40,6 +40,7 @@
 
 #include "download/download_info.h"
 #include "download/download_main.h"
+#include "net/address_list.h"
 #include "protocol/peer_connection_base.h"
 #include "torrent/exceptions.h"
 #include "torrent/peer/peer_info.h"
@@ -55,7 +56,7 @@ ConnectionList::clear() {
 }
 
 PeerConnectionBase*
-ConnectionList::insert(PeerInfo* peerInfo, const SocketFd& fd, Bitfield* bitfield, EncryptionInfo* encryptionInfo) {
+ConnectionList::insert(PeerInfo* peerInfo, const SocketFd& fd, Bitfield* bitfield, EncryptionInfo* encryptionInfo, ProtocolExtension* extensions) {
   if (size() >= m_maxSize)
     return NULL;
 
@@ -65,7 +66,8 @@ ConnectionList::insert(PeerInfo* peerInfo, const SocketFd& fd, Bitfield* bitfiel
     throw internal_error("ConnectionList::insert(...) received a NULL pointer.");
 
   peerInfo->set_connection(peerConnection);
-  peerConnection->initialize(m_download, peerInfo, fd, bitfield, encryptionInfo);
+  peerInfo->set_last_connection(cachedTime.seconds());
+  peerConnection->initialize(m_download, peerInfo, fd, bitfield, encryptionInfo, extensions);
 
   base_type::push_back(peerConnection);
 
@@ -94,7 +96,7 @@ ConnectionList::erase(iterator pos, int flags) {
   peerConnection->cleanup();
   peerConnection->peer_info()->set_connection(NULL);
 
-  m_download->peer_list()->disconnected(peerConnection->peer_info(), 0);
+  m_download->peer_list()->disconnected(peerConnection->peer_info(), PeerList::disconnect_set_time);
 
   // Delete after the signal to ensure the address of 'v' doesn't get
   // allocated for a different PCB in the signal.
