@@ -75,6 +75,9 @@ public:
     m_isActive(false),
     m_isCompact(true),
     m_isAcceptingNewPeers(true),
+    m_isPrivate(false),
+    m_pexEnabled(true),
+    m_pexActive(true),
 
     m_key(0),
     m_numwant(-1),
@@ -84,7 +87,9 @@ public:
     m_skipRate(60),
 
     m_uploadedBaseline(0),
-    m_completedBaseline(0) {
+    m_completedBaseline(0),
+    m_sizePex(0),
+    m_maxSizePex(8) {
   }
 
   const std::string&  name() const                                 { return m_name; }
@@ -110,7 +115,16 @@ public:
 
   bool                is_accepting_new_peers() const               { return m_isAcceptingNewPeers; }
   void                set_accepting_new_peers(bool s)              { m_isAcceptingNewPeers = s; }
-  
+
+  bool                is_private() const                           { return m_isPrivate; }
+  void                set_private(bool p)                          { m_isPrivate = p; if (p) m_pexEnabled = false; }
+
+  bool                is_pex_enabled() const                       { return m_pexEnabled; }
+  void                set_pex_enabled(bool enabled)                { m_pexEnabled = enabled && !m_isPrivate; }
+
+  bool                is_pex_active() const                        { return m_pexActive; }
+  void                set_pex_active(bool active)                  { m_pexActive = active; }
+
   uint32_t            key() const                                  { return m_key; }
   void                set_key(uint32_t key)                        { m_key = key; }
 
@@ -127,6 +141,14 @@ public:
   uint64_t            completed_baseline() const                   { return m_completedBaseline; }
   void                set_completed_baseline(uint64_t b)           { m_completedBaseline = b; }
 
+  uint32_t            size_pex() const                             { return m_sizePex; }
+  void                set_size_pex(uint32_t b)                     { m_sizePex = b; }
+
+  uint32_t            max_size_pex() const                         { return m_maxSizePex; }
+  void                set_max_size_pex(uint32_t b)                 { m_maxSizePex = b; }
+
+  uint32_t            max_size_pex_list() const                    { return 200; }
+
   uint32_t            http_timeout() const                         { return 60; }
   uint32_t            udp_timeout() const                          { return 30; }
   uint32_t            udp_tries() const                            { return 2; }
@@ -144,10 +166,14 @@ private:
   HashString          m_hashObfuscated;
   HashString          m_localId;
 
+  // TODO: Use flags.
   bool                m_isOpen;
   bool                m_isActive;
   bool                m_isCompact;
   bool                m_isAcceptingNewPeers;
+  bool                m_isPrivate;
+  bool                m_pexEnabled;
+  bool                m_pexActive;
 
   uint32_t            m_key;
   int32_t             m_numwant;
@@ -158,6 +184,8 @@ private:
 
   uint64_t            m_uploadedBaseline;
   uint64_t            m_completedBaseline;
+  uint32_t            m_sizePex;
+  uint32_t            m_maxSizePex;
 
   slot_stat_type      m_slotStatCompleted;
   slot_stat_type      m_slotStatLeft;
