@@ -40,6 +40,7 @@
 #include <limits>
 
 #include "data/chunk_list.h"
+#include "protocol/extensions.h"
 #include "protocol/handshake_manager.h"
 #include "protocol/peer_connection_base.h"
 #include "tracker/tracker_manager.h"
@@ -112,6 +113,12 @@ DownloadMain::~DownloadMain() {
   delete m_chunkList;
   delete m_chunkSelector;
   delete m_info;
+
+  m_ut_pex_delta.clear();
+  m_ut_pex_initial.clear();
+
+  if (m_info->size_pex() != 0)
+    throw internal_error("DownloadMain::~DownloadMain(): m_info->size_pex() != 0.");
 }
 
 void
@@ -219,15 +226,17 @@ DownloadMain::receive_connect_peers() {
   if (!info()->is_active())
     return;
 
-  AvailableList::AddressList* alist = peer_list()->available_list()->buffer();
+  // TODO: Is this actually going to be used?
+  AddressList* alist = peer_list()->available_list()->buffer();
 
   if (!alist->empty()) {
     alist->sort();
-    peer_list()->available_list()->insert(alist);
+    peer_list()->insert_available(alist);
     alist->clear();
   }
 
   while (!peer_list()->available_list()->empty() &&
+         manager->connection_manager()->can_connect() &&
          connection_list()->size() < connection_list()->get_min_size() &&
          connection_list()->size() + m_slotCountHandshakes(this) < connection_list()->get_max_size()) {
     rak::socket_address sa = peer_list()->available_list()->pop_random();
@@ -251,14 +260,130 @@ DownloadMain::receive_tracker_request() {
   if (connection_list()->size() >= connection_list()->get_min_size())
     return;
 
-  if (connection_list()->size() < m_lastConnectedSize + 10 ||
-      !m_trackerManager->request_current())
-    // Try the next tracker if we couldn't get enough peers from the
-    // current one, or if we have connected more than
-    // TrackerManager::max_num_request times.
+  if (m_info->is_pex_enabled() || connection_list()->size() < m_lastConnectedSize + 10)
+    m_trackerManager->request_next();
+  else if (!m_trackerManager->request_current())
     m_trackerManager->request_next();
 
   m_lastConnectedSize = connection_list()->size();
 }
 
+struct SocketAddressCompact_less {
+  bool operator () (const SocketAddressCompact& a, const SocketAddressCompact& b) const {
+    return (a.addr < b.addr) || ((a.addr == b.addr) && (a.port < b.port));
+  }
+};
+
+void
+DownloadMain::do_peer_exchange() {
+  if (!info()->is_active())
+    throw internal_error("DownloadMain::do_peer_exchange called on inactive download.");
+
+  // Check whether we should tell the peers to stop/start sending PEX
+  // messages.
+  int togglePex = 0;
+
+  if (!m_info->is_pex_active() &&
+      m_connectionList->size() < m_connectionList->get_min_size() / 2 &&
+      m_peerList.available_list()->size() < m_peerList.available_list()->max_size() / 4) {
+    m_info->set_pex_active(true);
+
+    // Only set PEX_ENABLE if we don't have max_size_pex set to zero.
+    if (m_info->size_pex() < m_info->max_size_pex())
+      togglePex = PeerConnectionBase::PEX_ENABLE;
+
+  } else if (m_info->is_pex_active() &&
+             m_connectionList->size() >= m_connectionList->get_min_size()) {
+//              m_peerList.available_list()->size() >= m_peerList.available_list()->max_size() / 2) {
+    togglePex = PeerConnectionBase::PEX_DISABLE;
+    m_info->set_pex_active(false);
+  }
+
+  // Return if we don't really want to do anything?
+
+  ProtocolExtension::PEXList current;
+
+  for (ConnectionList::iterator itr = m_connectionList->begin(); itr != m_connectionList->end(); ++itr) {
+    const rak::socket_address* sa = rak::socket_address::cast_from((*itr)->peer_info()->socket_address());
+
+    if ((*itr)->peer_info()->listen_port() != 0 && sa->family() == rak::socket_address::af_inet)
+      current.push_back(SocketAddressCompact(sa->sa_inet()->address_n(), (*itr)->peer_info()->listen_port()));
+
+    if (!(*itr)->extensions()->is_remote_supported(ProtocolExtension::UT_PEX))
+      continue;
+
+    if (togglePex == PeerConnectionBase::PEX_ENABLE) {
+      (*itr)->set_peer_exchange(true);
+
+      if (m_info->size_pex() >= m_info->max_size_pex())
+        togglePex = 0;
+
+    } else if (!(*itr)->extensions()->is_local_enabled(ProtocolExtension::UT_PEX)) {
+      continue;
+
+    } else if (togglePex == PeerConnectionBase::PEX_DISABLE) {
+      (*itr)->set_peer_exchange(false);
+
+      continue;
+    }
+
+    // Still using the old buffer? Make a copy in this rare case.
+    DataBuffer* message = (*itr)->extension_message();
+
+    if (!message->empty() && (message->data() == m_ut_pex_initial.data() || message->data() == m_ut_pex_delta.data())) {
+      char* buffer = new char[message->length()];
+      memcpy(buffer, message->data(), message->length());
+      message->set(buffer, buffer + message->length(), true);
+    }
+
+    (*itr)->do_peer_exchange();
+  }
+
+  std::sort(current.begin(), current.end(), SocketAddressCompact_less());
+
+  ProtocolExtension::PEXList added;
+  added.reserve(current.size());
+  std::set_difference(current.begin(), current.end(), m_ut_pex_list.begin(), m_ut_pex_list.end(), 
+                      std::back_inserter(added), SocketAddressCompact_less());
+
+  ProtocolExtension::PEXList removed;
+  removed.reserve(m_ut_pex_list.size());
+  std::set_difference(m_ut_pex_list.begin(), m_ut_pex_list.end(), current.begin(), current.end(), 
+                      std::back_inserter(removed), SocketAddressCompact_less());
+    
+  if (current.size() > m_info->max_size_pex_list()) {
+    // This test is only correct as long as we have a constant max
+    // size.
+    if (added.size() < current.size() - m_info->max_size_pex_list())
+      throw internal_error("DownloadMain::do_peer_exchange() added.size() < current.size() - m_info->max_size_pex_list().");
+
+    // Randomize this:
+    added.erase(added.end() - (current.size() - m_info->max_size_pex_list()), added.end());
+
+    // Create the new m_ut_pex_list by removing any 'removed'
+    // addresses from the original list and then adding the new
+    // addresses.
+    m_ut_pex_list.erase(std::set_difference(m_ut_pex_list.begin(), m_ut_pex_list.end(), removed.begin(), removed.end(),
+                                            m_ut_pex_list.begin(), SocketAddressCompact_less()), m_ut_pex_list.end());
+    m_ut_pex_list.insert(m_ut_pex_list.end(), added.begin(), added.end());
+
+    std::sort(m_ut_pex_list.begin(), m_ut_pex_list.end(), SocketAddressCompact_less());
+
+  } else {
+    m_ut_pex_list.swap(current);
+  }
+
+  current.clear();
+  m_ut_pex_delta.clear();
+
+  // If no peers were added or removed, the initial message is still correct and
+  // the delta message stays emptied. Otherwise generate the appropriate messages.
+  if (!added.empty() || !m_ut_pex_list.empty()) {
+    m_ut_pex_delta = ProtocolExtension::generate_ut_pex_message(added, removed);
+
+    m_ut_pex_initial.clear();
+    m_ut_pex_initial = ProtocolExtension::generate_ut_pex_message(m_ut_pex_list, current);
+  }
+}
+
 }
