@@ -39,7 +39,7 @@
 #include <iterator>
 #include <stdlib.h>
 #include <rak/file_stat.h>
-#include <sigc++/bind.h>
+#include <sigc++/adaptors/bind.h>
 
 #include "data/chunk_list.h"
 #include "data/hash_queue.h"
@@ -257,8 +257,7 @@ DownloadWrapper::receive_storage_error(const std::string& str) {
 
 void
 DownloadWrapper::receive_tracker_success(AddressList* l) {
-  m_main.connection_list()->set_difference(l);
-  m_main.peer_list()->available_list()->insert(l);
+  m_main.peer_list()->insert_available(l);
   m_main.receive_connect_peers();
   m_main.receive_tracker_success();
 
@@ -294,6 +293,18 @@ DownloadWrapper::receive_tick(uint32_t ticks) {
 
   // Every 2 minutes.
   if (ticks % 4 == 0) {
+    if (info()->is_active()) {
+      if (info()->is_pex_enabled()) {
+        m_main.do_peer_exchange();
+
+      // If PEX was disabled since the last peer exchange, deactivate it now.
+      } else if (info()->is_pex_active()) {
+        info()->set_pex_active(false);
+        for (ConnectionList::iterator itr = m_main.connection_list()->begin(); itr != m_main.connection_list()->end(); ++itr)
+          (*itr)->set_peer_exchange(false);
+      }
+    }
+
     for (ConnectionList::iterator itr = m_main.connection_list()->begin(); itr != m_main.connection_list()->end(); )
       if (!(*itr)->receive_keepalive())
         itr = m_main.connection_list()->erase(itr, ConnectionList::disconnect_available);
