@@ -38,6 +38,8 @@
 
 #include "download/download_info.h"
 #include "download/download_main.h"
+#include "net/throttle_list.h"
+#include "net/throttle_manager.h"
 #include "torrent/exceptions.h"
 #include "torrent/error.h"
 #include "torrent/poll.h"
@@ -46,6 +48,7 @@
 #include "globals.h"
 #include "manager.h"
 
+#include "extensions.h"
 #include "handshake.h"
 #include "handshake_manager.h"
 
@@ -78,12 +81,17 @@ Handshake::Handshake(SocketFd fd, HandshakeManager* m, int encryptionOptions) :
   m_peerInfo(NULL),
   m_download(NULL),
 
+  // Use global throttles until we know which download it is.
+  m_uploadThrottle(manager->upload_throttle()->throttle_list()),
+  m_downloadThrottle(manager->download_throttle()->throttle_list()),
+
   m_initializedTime(cachedTime),
 
   m_readDone(false),
   m_writeDone(false),
 
-  m_encryption(encryptionOptions) {
+  m_encryption(encryptionOptions),
+  m_extensions(m->default_extensions()) {
 
   set_fd(fd);
 
@@ -126,6 +134,9 @@ void
 Handshake::initialize_outgoing(const rak::socket_address& sa, DownloadMain* d, PeerInfo* peerInfo) {
   m_download = d;
 
+  m_uploadThrottle = d->upload_throttle();
+  m_downloadThrottle = d->download_throttle();
+
   m_peerInfo = peerInfo;
   m_peerInfo->set_flags(PeerInfo::flag_handshake);
 
@@ -175,6 +186,11 @@ Handshake::destroy_connection() {
 
   m_peerInfo->unset_flags(PeerInfo::flag_handshake);
   m_peerInfo = NULL;
+
+  if (!m_extensions->is_default()) {
+    m_extensions->cleanup();
+    delete m_extensions;
+  }
 }
 
 int
@@ -191,13 +207,23 @@ Handshake::retry_options() {
   return options;
 }
 
+inline uint32_t
+Handshake::read_unthrottled(void* buf, uint32_t length) {
+  return m_downloadThrottle->node_used_unthrottled(read_stream_throws(buf, length));
+}
+
+inline uint32_t
+Handshake::write_unthrottled(const void* buf, uint32_t length) {
+  return m_uploadThrottle->node_used_unthrottled(write_stream_throws(buf, length));
+}
+
 // Handshake::read_proxy_connect()
 // Entry: 0, 0
 // * 0, [0, 508>
 bool
 Handshake::read_proxy_connect() {
   // Being greedy for now.
-  m_readBuffer.move_end(read_stream_throws(m_readBuffer.end(), 512));
+  m_readBuffer.move_end(read_unthrottled(m_readBuffer.end(), 512));
   
   const char* pattern = "\r\n\r\n";
   const unsigned int patternLength = 4;
@@ -222,7 +248,7 @@ bool
 Handshake::read_encryption_key() {
   if (m_incoming) {
     if (m_readBuffer.remaining() < 20)
-      m_readBuffer.move_end(read_stream_throws(m_readBuffer.end(), 20 - m_readBuffer.remaining()));
+      m_readBuffer.move_end(read_unthrottled(m_readBuffer.end(), 20 - m_readBuffer.remaining()));
   
     if (m_readBuffer.remaining() < 20)
       return false;
@@ -241,7 +267,7 @@ Handshake::read_encryption_key() {
   // because peer can't send anything beyond the initial BT handshake
   // because it doesn't know our encryption choice yet.
   if (m_readBuffer.remaining() < enc_pad_read_size)
-    m_readBuffer.move_end(read_stream_throws(m_readBuffer.end(), enc_pad_read_size - m_readBuffer.remaining()));
+    m_readBuffer.move_end(read_unthrottled(m_readBuffer.end(), enc_pad_read_size - m_readBuffer.remaining()));
 
   // but we need at least the key at this point
   if (m_readBuffer.size_end() < 96)
@@ -288,7 +314,7 @@ Handshake::read_encryption_sync() {
     if (toRead <= 0)
       throw handshake_error(ConnectionManager::handshake_failed, e_handshake_encryption_sync_failed);
 
-    m_readBuffer.move_end(read_stream_throws(m_readBuffer.end(), toRead));
+    m_readBuffer.move_end(read_unthrottled(m_readBuffer.end(), toRead));
 
     itr = std::search(m_readBuffer.position(), m_readBuffer.end(),
                       (uint8_t*)m_encryption.sync(), (uint8_t*)m_encryption.sync() + m_encryption.sync_length());
@@ -322,6 +348,9 @@ Handshake::read_encryption_skey() {
 
   validate_download();
 
+  m_uploadThrottle = m_download->upload_throttle();
+  m_downloadThrottle = m_download->download_throttle();
+
   m_encryption.initialize_encrypt(m_download->info()->hash().c_str(), m_incoming);
   m_encryption.initialize_decrypt(m_download->info()->hash().c_str(), m_incoming);
 
@@ -466,6 +495,10 @@ Handshake::read_info() {
     }
 
     validate_download();
+
+    m_uploadThrottle = m_download->upload_throttle();
+    m_downloadThrottle = m_download->download_throttle();
+
     prepare_handshake();
 
   } else {
@@ -486,6 +519,10 @@ Handshake::read_peer() {
 
   prepare_peer_info();
 
+  // Send EXTENSION_PROTOCOL handshake message if peer supports it.
+  if (m_peerInfo->supports_extensions())
+    write_extension_handshake();
+
   // The download is just starting so we're not sending any
   // bitfield.
   if (m_download->file_list()->bitfield()->is_all_unset())
@@ -493,58 +530,20 @@ Handshake::read_peer() {
   else
     prepare_bitfield();
 
-  m_state = BITFIELD;
+  m_state = READ_MESSAGE;
   manager->poll()->insert_write(this);
 
   // Give some extra time for reading/writing the bitfield.
   priority_queue_erase(&taskScheduler, &m_taskTimeout);
   priority_queue_insert(&taskScheduler, &m_taskTimeout, (cachedTime + rak::timer::from_seconds(120)).round_seconds());
 
-  return false;
+  return true;
 }
 
 bool
 Handshake::read_bitfield() {
-  Bitfield* bitfield = m_download->file_list()->mutable_bitfield();
-
-  if (m_bitfield.empty()) {
-    fill_read_buffer(5);
-
-    // Received a keep-alive message which means we won't be
-    // getting any bitfield.
-    if (m_readBuffer.remaining() >= 4 && m_readBuffer.peek_32() == 0) {
-      m_readBuffer.read_32();
-      read_done();
-      return false;
-    }
-
-    if (m_readBuffer.remaining() < 5)
-      return false;
-
-    // Received a non-bitfield command.
-    if (m_readBuffer.peek_8_at(4) != protocol_bitfield) {
-      read_done();
-      return false;
-    }
-
-    if (m_readBuffer.read_32() != bitfield->size_bytes() + 1)
-      throw handshake_error(ConnectionManager::handshake_failed, e_handshake_invalid_value);
-
-    m_readBuffer.read_8();
-    m_readPos = 0;
-
-    m_bitfield.set_size_bits(bitfield->size_bits());
-    m_bitfield.allocate();
-
-    if (m_readBuffer.remaining() != 0) {
-      m_readPos = std::min<uint32_t>(m_bitfield.size_bytes(), m_readBuffer.remaining());
-      std::memcpy(m_bitfield.begin(), m_readBuffer.position(), m_readPos);
-      m_readBuffer.consume(m_readPos);
-    }
-  }
-
   if (m_readPos < m_bitfield.size_bytes()) {
-    uint32_t length = read_stream_throws(m_bitfield.begin() + m_readPos, m_bitfield.size_bytes() - m_readPos);
+    uint32_t length = read_unthrottled(m_bitfield.begin() + m_readPos, m_bitfield.size_bytes() - m_readPos);
 
     if (m_encryption.info()->decrypt_valid())
       m_encryption.info()->decrypt(m_bitfield.begin() + m_readPos, length);
@@ -552,12 +551,75 @@ Handshake::read_bitfield() {
     m_readPos += length;
   }
 
-  if (m_readPos == m_bitfield.size_bytes())
-    read_done();
+  return m_readPos == m_bitfield.size_bytes();
+}
 
+bool
+Handshake::read_extension() {
+  if (m_readBuffer.peek_32() > m_readBuffer.reserved())
+    throw handshake_error(ConnectionManager::handshake_failed, e_handshake_invalid_value);
+
+  int32_t need = m_readBuffer.peek_32() + 4 - m_readBuffer.remaining();
+
+  // We currently can't handle an extension handshake that doesn't
+  // completely fit in the buffer.  However these messages are usually
+  // ~100 bytes large and the buffer holds over 1000 bytes so it
+  // should be ok. Else maybe figure out how to disable extensions for
+  // when peer connects next time.
+  //
+  // In addition, make sure there's at least 5 bytes available after
+  // the PEX message has been read, so that we can fit the preamble of
+  // the BITFIELD message.
+  if (need + 5 > m_readBuffer.reserved_left()) {
+    m_readBuffer.move_unused();
+
+    if (need + 5 > m_readBuffer.reserved_left())
+      throw handshake_error(ConnectionManager::handshake_failed, e_handshake_invalid_value);
+  }
+
+  if (!fill_read_buffer(m_readBuffer.peek_32() + 4))
+    return false;
+
+  uint32_t length = m_readBuffer.read_32() - 2;
+  m_readBuffer.read_8();
+  m_extensions->read_start(m_readBuffer.read_8(), length, false);
+
+  std::memcpy(m_extensions->read_position(), m_readBuffer.position(), length);
+  m_extensions->read_move(length);
+
+  // Does this check need to check if it is a handshake we read?
+  if (!m_extensions->is_complete())
+    throw internal_error("Could not read extension handshake even though it should be in the read buffer.");
+
+  m_extensions->read_done();
+  m_readBuffer.consume(length);
   return true;
 }
 
+void
+Handshake::read_done() {
+  if (m_readDone != false)
+    throw internal_error("Handshake::read_done() m_readDone != false.");
+
+//   if (m_peerInfo->supports_extensions() && m_extensions->is_initial_handshake())
+//     throw handshake_error(ConnectionManager::handshake_failed, e_handshake_invalid_order);
+
+  m_readDone = true;
+  manager->poll()->remove_read(this);
+
+  if (m_bitfield.empty()) {
+    m_bitfield.set_size_bits(m_download->file_list()->bitfield()->size_bits());
+    m_bitfield.allocate();
+    m_bitfield.unset_all();
+
+  } else {
+    m_bitfield.update();
+  }
+
+  if (m_writeDone)
+    throw handshake_succeeded();
+}
+
 void
 Handshake::event_read() {
   try {
@@ -640,12 +702,73 @@ restart:
       if (!read_peer())
         break;
 
-      if (m_state != BITFIELD)
+      // Is this correct?
+      if (m_state != READ_MESSAGE)
         goto restart;
 
-    case BITFIELD:
-      read_bitfield();
-      break;
+    case READ_MESSAGE:
+      fill_read_buffer(5);
+
+      // Received a keep-alive message which means we won't be
+      // getting any bitfield.
+      if (m_readBuffer.remaining() >= 4 && m_readBuffer.peek_32() == 0) {
+        m_readBuffer.read_32();
+        read_done();
+        break;
+      }
+
+      if (m_readBuffer.remaining() < 5)
+        break;
+
+      m_readPos = 0;
+
+      // Extension handshake was sent after BT handshake but before
+      // bitfield, so handle that. If we've already received a message
+      // of this type then we will assume the peer won't be sending a
+      // bitfield, as the second extension message will be part of the
+      // normal traffic, not the handshake.
+      if (m_readBuffer.peek_8_at(4) == protocol_bitfield) {
+        const Bitfield* bitfield = m_download->file_list()->bitfield();
+
+        if (!m_bitfield.empty() || m_readBuffer.read_32() != bitfield->size_bytes() + 1)
+          throw handshake_error(ConnectionManager::handshake_failed, e_handshake_invalid_value);
+
+        m_readBuffer.read_8();
+
+        m_bitfield.set_size_bits(bitfield->size_bits());
+        m_bitfield.allocate();
+
+        m_readPos = std::min<uint32_t>(m_bitfield.size_bytes(), m_readBuffer.remaining());
+        std::memcpy(m_bitfield.begin(), m_readBuffer.position(), m_readPos);
+        m_readBuffer.consume(m_readPos);
+
+        m_state = READ_BITFIELD;
+
+      } else if (m_readBuffer.peek_8_at(4) == protocol_extension && m_extensions->is_initial_handshake()) {
+        m_readPos = 0;
+        m_state = READ_EXT;
+
+      } else {
+        read_done();
+        break;
+      }
+
+    case READ_BITFIELD:
+    case READ_EXT:
+      // Gather the different command types into the same case group
+      // so that we don't need 'goto restart' above.
+      if ((m_state == READ_BITFIELD && !read_bitfield()) ||
+          (m_state == READ_EXT && !read_extension()))
+        break;
+
+      m_state = READ_MESSAGE;
+
+      if (!m_bitfield.empty() && (!m_peerInfo->supports_extensions() || !m_extensions->is_initial_handshake())) {
+        read_done();
+        break;
+      }
+
+      goto restart;
 
     default:
       throw internal_error("Handshake::event_read() called in invalid state.");
@@ -653,7 +776,7 @@ restart:
 
     // Call event_write if we have any data to write. Make sure
     // event_write() doesn't get called twice in this function.
-    if (m_writeBuffer.remaining()) {
+    if (m_writeBuffer.remaining() && !manager->poll()->in_write(this)) {
       manager->poll()->insert_write(this);
       return event_write();
     }
@@ -675,7 +798,7 @@ Handshake::fill_read_buffer(int size) {
     if (size - m_readBuffer.remaining() > m_readBuffer.reserved_left())
       throw internal_error("Handshake::fill_read_buffer(...) Buffer overflow.");
 
-    int read = m_readBuffer.move_end(read_stream_throws(m_readBuffer.end(), size - m_readBuffer.remaining()));
+    int read = m_readBuffer.move_end(read_unthrottled(m_readBuffer.end(), size - m_readBuffer.remaining()));
 
     if (m_encryption.info()->decrypt_valid())
       m_encryption.info()->decrypt(m_readBuffer.end() - read, read);
@@ -694,27 +817,6 @@ Handshake::validate_download() {
     throw handshake_error(ConnectionManager::handshake_dropped, e_handshake_not_accepting_connections);
 }
 
-void
-Handshake::read_done() {
-  if (m_readDone != false)
-    throw internal_error("Handshake::read_done() m_readDone != false.");
-
-  m_readDone = true;
-  manager->poll()->remove_read(this);
-
-  if (m_bitfield.empty()) {
-    m_bitfield.set_size_bits(m_download->file_list()->bitfield()->size_bits());
-    m_bitfield.allocate();
-    m_bitfield.unset_all();
-
-  } else {
-    m_bitfield.update();
-  }
-
-  if (m_writeDone)
-    throw handshake_succeeded();
-}
-
 void
 Handshake::event_write() {
   try {
@@ -722,7 +824,7 @@ Handshake::event_write() {
     switch (m_state) {
     case CONNECTING:
       if (get_fd().get_error())
-        throw handshake_error(ConnectionManager::handshake_failed, e_handshake_network_error);
+        throw handshake_error(ConnectionManager::handshake_failed, e_handshake_network_unreachable);
 
       manager->poll()->insert_read(this);
 
@@ -766,7 +868,9 @@ Handshake::event_write() {
 
       break;
 
-    case BITFIELD:
+    case READ_MESSAGE:
+    case READ_BITFIELD:
+    case READ_EXT:
       write_bitfield();
       return;
 
@@ -777,7 +881,7 @@ Handshake::event_write() {
     if (!m_writeBuffer.remaining())
       throw internal_error("event_write called with empty write buffer.");
 
-    if (m_writeBuffer.consume(write_stream_throws(m_writeBuffer.position(), m_writeBuffer.remaining())))
+    if (m_writeBuffer.consume(write_unthrottled(m_writeBuffer.position(), m_writeBuffer.remaining())))
       manager->poll()->remove_write(this);
 
   } catch (handshake_succeeded& e) {
@@ -861,6 +965,7 @@ Handshake::prepare_handshake() {
   m_writeBuffer.write_range(m_protocol, m_protocol + 19);
 
   std::memset(m_writeBuffer.end(), 0, 8);
+  *(m_writeBuffer.end()+5) |= 0x10;    // support extension protocol
   m_writeBuffer.move_end(8);
 
   m_writeBuffer.write_range(m_download->info()->hash().c_str(), m_download->info()->hash().c_str() + 20);
@@ -916,6 +1021,32 @@ Handshake::prepare_keepalive() {
   m_writePos = m_download->file_list()->bitfield()->size_bytes();
 }
 
+void
+Handshake::write_extension_handshake() {
+  DownloadInfo* info = m_download->info();
+
+  if (m_extensions->is_default()) {
+    m_extensions = new ProtocolExtension;
+    m_extensions->set_info(m_peerInfo, m_download);
+  }
+
+  // PEX may be disabled but still active if disabled since last download tick.
+  if (info->is_pex_enabled() && info->is_pex_active() && info->size_pex() < info->max_size_pex())
+    m_extensions->set_local_enabled(ProtocolExtension::UT_PEX);
+
+  DataBuffer message = m_extensions->generate_handshake_message();
+
+  m_writeBuffer.write_32(message.length() + 2);
+  m_writeBuffer.write_8(protocol_extension);
+  m_writeBuffer.write_8(ProtocolExtension::HANDSHAKE);
+  m_writeBuffer.write_range(message.data(), message.end());
+
+  if (m_encryption.info()->is_encrypted())
+    m_encryption.info()->encrypt(m_writeBuffer.end() - message.length() - 2 - 4, message.length() + 2 + 4);
+
+  message.clear();
+}
+
 void
 Handshake::write_bitfield() {
   const Bitfield* bitfield = m_download->file_list()->bitfield();
@@ -924,7 +1055,7 @@ Handshake::write_bitfield() {
     throw internal_error("Handshake::event_write() m_writeDone != false.");
 
   if (m_writeBuffer.remaining())
-    if (!m_writeBuffer.consume(write_stream_throws(m_writeBuffer.position(), m_writeBuffer.remaining())))
+    if (!m_writeBuffer.consume(write_unthrottled(m_writeBuffer.position(), m_writeBuffer.remaining())))
       return;
 
   if (m_writePos != bitfield->size_bytes()) {
@@ -940,7 +1071,7 @@ Handshake::write_bitfield() {
         m_writeBuffer.move_end(length);
       }
 
-      length = write_stream_throws(m_writeBuffer.begin(), m_writeBuffer.size_end());
+      length = write_unthrottled(m_writeBuffer.begin(), m_writeBuffer.size_end());
       m_writePos += length;
 
       if (length != m_writeBuffer.size_end() && length > 0)
@@ -949,8 +1080,8 @@ Handshake::write_bitfield() {
       m_writeBuffer.move_end(-length);
 
     } else {
-      m_writePos += write_stream_throws(bitfield->begin() + m_writePos,
-                                        bitfield->size_bytes() - m_writePos);
+      m_writePos += write_unthrottled(bitfield->begin() + m_writePos,
+                                      bitfield->size_bytes() - m_writePos);
     }
   }
 
