@@ -58,7 +58,9 @@ public:
   static const uint32_t part2_size     = 20;
   static const uint32_t handshake_size = part1_size + part2_size;
 
-  static const uint32_t protocol_bitfield = 5;
+  static const uint32_t protocol_bitfield  = 5;
+  static const uint32_t protocol_port      = 9;
+  static const uint32_t protocol_extension = 20;
 
   static const uint32_t enc_negotiation_size = 8 + 4 + 2;
   static const uint32_t enc_pad_size         = 512;
@@ -71,7 +73,6 @@ public:
   typedef enum {
     INACTIVE,
     CONNECTING,
-    BITFIELD,
 
     PROXY_CONNECT,
     PROXY_DONE,
@@ -85,6 +86,9 @@ public:
 
     READ_INFO,
     READ_PEER,
+    READ_MESSAGE,
+    READ_BITFIELD,
+    READ_EXT
   } State;
 
   Handshake(SocketFd fd, HandshakeManager* m, int encryption_options);
@@ -92,6 +96,8 @@ public:
 
   bool                is_active() const             { return m_state != INACTIVE; }
 
+  State               state() const                 { return m_state; }
+
   void                initialize_incoming(const rak::socket_address& sa);
   void                initialize_outgoing(const rak::socket_address& sa, DownloadMain* d, PeerInfo* peerInfo);
   
@@ -118,7 +124,8 @@ public:
   virtual void        event_write();
   virtual void        event_error();
 
-  HandshakeEncryption* encryption() { return &m_encryption; }
+  HandshakeEncryption* encryption()                 { return &m_encryption; }
+  ProtocolExtension*   extensions()                  { return m_extensions; }
 
   int                 retry_options();
 
@@ -140,6 +147,7 @@ protected:
   bool                read_info();
   bool                read_peer();
   bool                read_bitfield();
+  bool                read_extension();
 
   void                prepare_proxy_connect();
   void                prepare_key_plus_pad();
@@ -149,10 +157,14 @@ protected:
   void                prepare_bitfield();
   void                prepare_keepalive();
 
+  void                write_extension_handshake();
   void                write_bitfield();
 
   inline void         validate_download();
 
+  uint32_t            read_unthrottled(void* buf, uint32_t length);
+  uint32_t            write_unthrottled(const void* buf, uint32_t length);
+
   static const char*  m_protocol;
 
   State               m_state;
@@ -163,6 +175,9 @@ protected:
   DownloadMain*       m_download;
   Bitfield            m_bitfield;
 
+  ThrottleList*       m_uploadThrottle;
+  ThrottleList*       m_downloadThrottle;
+
   rak::priority_item  m_taskTimeout;
   rak::timer          m_initializedTime;
 
@@ -178,6 +193,7 @@ protected:
   char                m_options[8];
 
   HandshakeEncryption m_encryption;
+  ProtocolExtension*  m_extensions;
 
   // Put these last to keep variables closer to *this.
   Buffer              m_readBuffer;
