@@ -54,6 +54,8 @@
 
 namespace torrent {
 
+ProtocolExtension HandshakeManager::DefaultExtensions = ProtocolExtension::make_default();
+
 inline void
 handshake_manager_delete_handshake(Handshake* h) {
   h->deactivate_connection();
@@ -131,7 +133,8 @@ HandshakeManager::add_outgoing(const rak::socket_address& sa, DownloadMain* down
 
 void
 HandshakeManager::create_outgoing(const rak::socket_address& sa, DownloadMain* download, int encryptionOptions) {
-  PeerInfo* peerInfo = download->peer_list()->connected(sa.c_sockaddr(), PeerList::connect_keep_handshakes);
+  PeerInfo* peerInfo = download->peer_list()->connected(sa.c_sockaddr(),
+                                                        PeerList::connect_keep_handshakes | PeerList::connect_filter_recent);
 
   if (peerInfo == NULL)
     return;
@@ -193,10 +196,11 @@ HandshakeManager::receive_succeeded(Handshake* handshake) {
       (!download->file_list()->is_done() || !handshake->bitfield()->is_all_set()) &&
 
       (pcb = download->connection_list()->insert(handshake->peer_info(),
-                                                              handshake->get_fd(),
-                                                              handshake->bitfield(),
-                                                              handshake->encryption()->info())) != NULL) {
-
+                                                 handshake->get_fd(),
+                                                 handshake->bitfield(),
+                                                 handshake->encryption()->info(),
+                                                 handshake->extensions())) != NULL) {
+    
     manager->client_list()->retrieve_id(&handshake->peer_info()->mutable_client_info(), handshake->peer_info()->id());
     manager->connection_manager()->signal_handshake_log().emit(handshake->peer_info()->socket_address(),
                                                                ConnectionManager::handshake_success,
@@ -269,7 +273,7 @@ HandshakeManager::receive_failed(Handshake* handshake, int message, int error) {
 
 void
 HandshakeManager::receive_timeout(Handshake* h) {
-  receive_failed(h, ConnectionManager::handshake_failed, e_handshake_network_error);
+  receive_failed(h, ConnectionManager::handshake_failed, h->state() == Handshake::CONNECTING ? e_handshake_network_unreachable : e_handshake_network_timeout);
 }
 
 bool
