@@ -46,6 +46,7 @@
 #include "download/download_info.h"
 #include "download/download_main.h"
 
+#include "extensions.h"
 #include "peer_connection_leech.h"
 
 namespace torrent {
@@ -249,6 +250,7 @@ PeerConnectionLeech::read_message() {
 
       } else {
         m_down->set_state(ProtocolRead::READ_SKIP_PIECE);
+        m_download->download_throttle()->insert(m_peerChunks.download_throttle());
         return false;
       }
       
@@ -273,6 +275,26 @@ PeerConnectionLeech::read_message() {
     read_cancel_piece(m_down->read_request());
     return true;
 
+  case ProtocolBase::EXTENSION_PROTOCOL:
+    if (!m_down->can_read_extension_body())
+      break;
+
+    if (m_extensions->is_default()) {
+      m_extensions = new ProtocolExtension();
+      m_extensions->set_info(m_peerInfo, m_download);
+    }
+
+    {
+      int extension = m_down->buffer()->read_8();
+      m_extensions->read_start(extension, length - 2, (extension == ProtocolExtension::UT_PEX) && !m_download->want_pex_msg());
+      m_down->set_state(ProtocolRead::READ_EXTENSION);
+    }
+
+    if (down_extension())
+      m_down->set_state(ProtocolRead::IDLE);
+
+    return true;
+
   default:
     throw communication_error("Received unsupported message type.");
   }
@@ -308,6 +330,7 @@ PeerConnectionLeech::event_read() {
       case ProtocolRead::IDLE:
         if (m_down->buffer()->size_end() < read_size) {
           unsigned int length = read_stream_throws(m_down->buffer()->end(), read_size - m_down->buffer()->size_end());
+          m_download->download_throttle()->node_used_unthrottled(length);
 
           if (is_encrypted())
             m_encryption.decrypt(m_down->buffer()->end(), length);
@@ -345,7 +368,6 @@ PeerConnectionLeech::event_read() {
       case ProtocolRead::READ_SKIP_PIECE:
         if (download_queue()->transfer()->is_leader()) {
           m_down->set_state(ProtocolRead::READ_PIECE);
-          m_download->download_throttle()->insert(m_peerChunks.download_throttle());
           break;
         }
 
@@ -357,6 +379,13 @@ PeerConnectionLeech::event_read() {
         down_chunk_finished();
         break;
 
+      case ProtocolRead::READ_EXTENSION:
+        if (!down_extension())
+          return;
+
+        m_down->set_state(ProtocolRead::IDLE);
+        break;
+
       default:
         throw internal_error("PeerConnectionLeech::event_read() wrong state.");
       }
@@ -458,9 +487,12 @@ PeerConnectionLeech::fill_write_buffer() {
     m_peerChunks.cancel_queue()->pop_front();
   }
 
-  if (!m_upChoke.choked() &&
-      !m_peerChunks.upload_queue()->empty() &&
-      m_up->can_write_piece())
+  if (m_sendPEXMask && m_up->can_write_extension())
+    send_pex_message();
+
+  else if (!m_upChoke.choked() &&
+           !m_peerChunks.upload_queue()->empty() &&
+           m_up->can_write_piece())
     write_prepare_piece();
 
   if (is_encrypted())
@@ -486,29 +518,41 @@ PeerConnectionLeech::event_write() {
         m_up->set_state(ProtocolWrite::MSG);
 
       case ProtocolWrite::MSG:
-        if (!m_up->buffer()->consume(write_stream_throws(m_up->buffer()->position(), m_up->buffer()->remaining())))
+        if (!m_up->buffer()->consume(m_download->upload_throttle()->node_used_unthrottled(write_stream_throws(m_up->buffer()->position(), m_up->buffer()->remaining()))))
           return;
 
         m_up->buffer()->reset();
 
-        if (m_up->last_command() != ProtocolBase::PIECE) {
+        if (m_up->last_command() == ProtocolBase::PIECE) {
+          // We're uploading a piece.
+          load_up_chunk();
+          m_up->set_state(ProtocolWrite::WRITE_PIECE);
+
+          // fall through to WRITE_PIECE case below
+
+        } else if (m_up->last_command() == ProtocolBase::EXTENSION_PROTOCOL) {
+          m_up->set_state(ProtocolWrite::WRITE_EXTENSION);
+          break;
+
+        } else {
           // Break or loop? Might do an ifelse based on size of the
           // write buffer. Also the write buffer is relatively large.
           m_up->set_state(ProtocolWrite::IDLE);
           break;
         }
 
-        // We're uploading a piece.
-        load_up_chunk();
-
-        m_up->set_state(ProtocolWrite::WRITE_PIECE);
-
       case ProtocolWrite::WRITE_PIECE:
         if (!up_chunk())
           return;
 
         m_up->set_state(ProtocolWrite::IDLE);
+        break;
 
+      case ProtocolWrite::WRITE_EXTENSION:
+        if (!up_extension())
+          return;
+
+        m_up->set_state(ProtocolWrite::IDLE);
         break;
 
       default:
