@@ -40,8 +40,8 @@
 #include <list>
 #include <vector>
 #include <string>
-#include <sigc++/slot.h>
 #include <sigc++/connection.h>
+#include <sigc++/functors/slot.h>
 
 #include <torrent/common.h>
 #include <torrent/peer/peer.h>
@@ -76,9 +76,15 @@ public:
   void                start();
   void                stop();
 
-//   static const int start_no_create = 0x1;
+  //   static const int start_no_create = (1 << 0);
+  static const int start_keep_baseline = (1 << 1);
+  static const int start_skip_tracker  = (1 << 2);
 
-//   void                start(int flags);
+  static const int stop_skip_tracker   = (1 << 0);
+
+  // These will be renamed when the API is updated.
+  void                start2(int flags);
+  void                stop2(int flags);
 
   // Does not check if the download has been removed.
   bool                is_valid() const { return m_ptr; }
@@ -89,6 +95,11 @@ public:
   bool                is_hash_checked() const;
   bool                is_hash_checking() const;
 
+  bool                is_private() const;
+  bool                is_pex_active() const;
+  bool                is_pex_enabled() const;
+  void                set_pex_enabled(bool enabled);
+
   // Returns "" if the object is not valid.
   const std::string&  name() const;
 
@@ -155,6 +166,9 @@ public:
   uint32_t            peers_currently_unchoked() const;
   uint32_t            peers_currently_interested() const;
 
+  uint32_t            size_pex() const;
+  uint32_t            max_size_pex() const;
+
   bool                accepting_new_peers() const;
   uint32_t            uploads_max() const;
   
