@@ -1,5 +1,5 @@
 // libTorrent - BitTorrent library
-// Copyright (C) 2005-2006, Jari Sundell
+// Copyright (C) 2005-2007, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@
 #include "config.h"
 
 #include <iterator>
-#include <sstream>
+#include <iostream>
 #include <rak/functional.h>
 
 #include "utils/sha1.h"
@@ -140,62 +140,19 @@ object_read_bencode(std::istream* input, Object* object, uint32_t depth) {
 
 void
 object_write_bencode(std::ostream* output, const Object* object) {
-  // A decent compiler should be able to optimize away the
-  // Object::check_type calls.
-  switch (object->type()) {
-  case Object::TYPE_NONE:
-    // Consider failing here?
-    break;
-
-  case Object::TYPE_VALUE:
-    *output << 'i' << object->as_value() << 'e';
-    break;
-
-  case Object::TYPE_STRING:
-    *output << object->as_string().size() << ':' << object->as_string();
-    break;
-
-  case Object::TYPE_LIST:
-    // Add check for depth here?
-    *output << 'l';
-
-    for (Object::list_type::const_iterator itr = object->as_list().begin(), last = object->as_list().end(); itr != last && output->good(); ++itr)
-      object_write_bencode(output, &*itr);
-
-    *output << 'e';
-    break;
-
-  case Object::TYPE_MAP:
-    // Add check for depth here?
-    *output << 'd';
-
-    for (Object::map_type::const_iterator itr = object->as_map().begin(), last = object->as_map().end(); itr != last && output->good(); ++itr) {
-      *output << itr->first.size() << ':' << itr->first;
-      object_write_bencode(output, &itr->second);
-    }
-
-    *output << 'e';
-    break;
-  }
+  char buffer[1024];
+  object_write_bencode_c(&object_write_to_stream, output, object_buffer_t(buffer, buffer + 1024), object);
 }
 
 // Would be nice to have a straight stream to hash conversion.
 std::string
 object_sha1(const Object* object) {
-  std::stringstream str;
-  str << *object;
+  Sha1 sha;
+  char buffer[1024];
 
-  if (str.fail())
-    throw bencode_error("Could not write bencode to stream");
-
-  std::string s = str.str();
-  Sha1 sha1;
-
-  sha1.init();
-  sha1.update(s.c_str(), s.size());
-
-  char buffer[20];
-  sha1.final_c(buffer);
+  sha.init();
+  object_write_bencode_c(&object_write_to_sha1, &sha, object_buffer_t(buffer, buffer + 1024), object);
+  sha.final_c(buffer);
 
   return std::string(buffer, 20);
 }
@@ -213,12 +170,155 @@ operator >> (std::istream& input, Object& object) {
 
 std::ostream&
 operator << (std::ostream& output, const Object& object) {
-  std::locale locale = output.imbue(std::locale::classic());
-
   object_write_bencode(&output, &object);
-
-  output.imbue(locale);
   return output;
 }
 
+struct object_write_data_t {
+  object_write_t writeFunc;
+  void* data;
+
+  object_buffer_t buffer;
+  char* pos;
+};
+
+void
+object_write_bencode_c_string(object_write_data_t* output, const char* srcData, uint32_t srcLength) {
+  do {
+    uint32_t len = std::min<uint32_t>(srcLength, std::distance(output->pos, output->buffer.second));
+
+    std::memcpy(output->pos, srcData, len);
+
+    output->pos += len;
+
+    if (output->pos == output->buffer.second) {
+      output->buffer = output->writeFunc(output->data, output->buffer);
+      output->pos = output->buffer.first;
+    }
+
+    srcData += len;
+    srcLength -= len;
+
+  } while (srcLength != 0);
+}
+
+void
+object_write_bencode_c_char(object_write_data_t* output, char src) {
+  if (output->pos == output->buffer.second) {
+    output->buffer = output->writeFunc(output->data, output->buffer);
+    output->pos = output->buffer.first;
+  }
+
+  *output->pos++ = src;
+}
+
+// A new wheel. Look, how shiny and new.
+void
+object_write_bencode_c_value(object_write_data_t* output, int64_t src) {
+  if (src == 0)
+    return object_write_bencode_c_char(output, '0');
+
+  if (src < 0) {
+    object_write_bencode_c_char(output, '-');
+    src = -src;
+  }
+
+  char buffer[20];
+  char* first = buffer + 20;
+
+  // We don't need locale support, so just do this directly.
+  while (src != 0) {
+    *--first = '0' + src % 10;
+
+    src /= 10;
+  }
+
+  object_write_bencode_c_string(output, first, 20 - std::distance(buffer, first));
+}
+
+void
+object_write_bencode_c_object(object_write_data_t* output, const Object* object) {
+  switch (object->type()) {
+  case Object::TYPE_NONE:
+    break;
+
+  case Object::TYPE_VALUE:
+    object_write_bencode_c_char(output, 'i');
+    object_write_bencode_c_value(output, object->as_value());
+    object_write_bencode_c_char(output, 'e');
+    break;
+
+  case Object::TYPE_STRING:
+    object_write_bencode_c_value(output, object->as_string().size());
+    object_write_bencode_c_char(output, ':');
+    object_write_bencode_c_string(output, object->as_string().c_str(), object->as_string().size());
+    break;
+
+  case Object::TYPE_LIST:
+    object_write_bencode_c_char(output, 'l');
+
+    for (Object::list_type::const_iterator itr = object->as_list().begin(), last = object->as_list().end(); itr != last; ++itr)
+      object_write_bencode_c_object(output, &*itr);
+
+    object_write_bencode_c_char(output, 'e');
+    break;
+
+  case Object::TYPE_MAP:
+    object_write_bencode_c_char(output, 'd');
+
+    for (Object::map_type::const_iterator itr = object->as_map().begin(), last = object->as_map().end(); itr != last; ++itr) {
+      object_write_bencode_c_value(output, itr->first.size());
+      object_write_bencode_c_char(output, ':');
+      object_write_bencode_c_string(output, itr->first.c_str(), itr->first.size());
+
+      object_write_bencode_c_object(output, &itr->second);
+    }
+
+    object_write_bencode_c_char(output, 'e');
+    break;
+  }
+}
+
+object_buffer_t
+object_write_bencode_c(object_write_t writeFunc, void* data, object_buffer_t buffer, const Object* object) {
+  object_write_data_t output;
+  output.writeFunc = writeFunc;
+  output.data      = data;
+  output.buffer    = buffer;
+  output.pos       = buffer.first;
+
+  object_write_bencode_c_object(&output, object);
+
+  // Don't flush the buffer.
+  if (output.pos == output.buffer.first)
+    return output.buffer;
+
+  return output.writeFunc(output.data, object_buffer_t(output.buffer.first, output.pos));
+}
+
+object_buffer_t
+object_write_to_buffer(void* data, object_buffer_t buffer) {
+  if (buffer.first == buffer.second)
+    throw internal_error("object_write_to_buffer(...) buffer overflow.");
+
+  return object_buffer_t(buffer.second, buffer.second);
+}
+
+object_buffer_t
+object_write_to_sha1(void* data, object_buffer_t buffer) {
+  reinterpret_cast<Sha1*>(data)->update(buffer.first, std::distance(buffer.first, buffer.second));
+
+  return buffer;
+}
+
+object_buffer_t
+object_write_to_stream(void* data, object_buffer_t buffer) {
+  reinterpret_cast<std::ostream*>(data)->write(buffer.first, std::distance(buffer.first, buffer.second));
+
+  if (reinterpret_cast<std::ostream*>(data)->bad())
+    return object_buffer_t(buffer.first, buffer.first);
+
+  return buffer;
+}
+
 }
