@@ -39,6 +39,9 @@
 #include <cstring>
 #include <rak/socket_address.h>
 
+#include "protocol/extensions.h"
+#include "protocol/peer_connection_base.h"
+
 #include "exceptions.h"
 #include "peer_info.h"
 
@@ -51,6 +54,7 @@ PeerInfo::PeerInfo(const sockaddr* address) :
   m_failedCounter(0),
   m_transferCounter(0),
   m_lastConnection(0),
+  m_lastHandshake(0),
   m_listenPort(0),
 
   m_connection(NULL)
