@@ -42,6 +42,8 @@
 
 namespace torrent {
 
+bool        socket_address_less(const sockaddr* s1, const sockaddr* s2);
+
 // Unique key for the address, excluding port numbers etc.
 class LIBTORRENT_EXPORT socket_address_key {
 public:
@@ -49,7 +51,7 @@ public:
 
   inline static bool is_comparable(const sockaddr* sa);
 
-  bool operator < (const socket_address_key& sa) const;
+  bool operator < (const socket_address_key& sa) const { return socket_address_less(m_sockaddr, sa.m_sockaddr); }
 
 private:
   const sockaddr*     m_sockaddr;
@@ -80,11 +82,13 @@ public:
 
   static const int connect_incoming        = (1 << 0);
   static const int connect_keep_handshakes = (1 << 1);
+  static const int connect_filter_recent   = (1 << 2);
 
   // Make sure any change here match ConnectionList's flags.
   static const int disconnect_available    = (1 << 0);
   static const int disconnect_quick        = (1 << 1);
   static const int disconnect_unwanted     = (1 << 2);
+  static const int disconnect_set_time     = (1 << 3);
 
   static const int cull_old                = (1 << 0);
   static const int cull_keep_interesting   = (1 << 1);
@@ -94,6 +98,9 @@ public:
 
   PeerInfo*           insert_address(const sockaddr* address, int flags);
 
+  // This will be used internally only for the moment.
+  uint32_t            insert_available(const void* al) LIBTORRENT_NO_EXPORT;
+
   AvailableList*      available_list()  { return m_availableList; }
 
   uint32_t            cull_peers(int flags);
