@@ -1,5 +1,5 @@
 // libTorrent - BitTorrent library
-// Copyright (C) 2005-2006, Jari Sundell
+// Copyright (C) 2005-2007, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -76,6 +76,16 @@ Tracker::tracker_type() const {
   return static_cast<Type>(m_tracker.second->type());
 }
 
+uint32_t
+Tracker::normal_interval() const {
+  return m_tracker.second->normal_interval();
+}
+
+uint32_t
+Tracker::min_interval() const {
+  return m_tracker.second->min_interval();
+}
+
 uint64_t
 Tracker::scrape_time_last() const {
   return m_tracker.second->scrape_time_last().usec();
