@@ -62,11 +62,6 @@ private:
   void                receive_done();
   void                receive_failed(std::string msg);
 
-  static rak::socket_address parse_address(const Object& b);
-
-  void                parse_address_normal(AddressList* l, const Object::list_type& b);
-  void                parse_address_compact(AddressList* l, const std::string& s);
-
   Http*               m_get;
   std::stringstream*  m_data;
 
